#!/usr/bin/env python

# Copyright 2005 PyTom <pytom@bishoujo.us>
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

import Image, ImageChops
import sys
import os.path
import re
import cStringIO

def read_spng(fn):
    f = file(fn, "rb")

    format = f.readline().strip()
    if format != "ALICE-SPNG-1":
        raise Exception("Invalid SPNG format.")

    width = int(f.readline())
    height = int(f.readline())
    xo = int(f.readline())
    yo = int(f.readline())

    im = Image.new("RGBA", (width, height))
    cim = Image.open(cStringIO.StringIO(f.read()))
    im.paste(cim, (xo, yo), cim)

    f.close()

    return im

    
dollspath = os.path.join(os.path.dirname(sys.argv[0]), "dolls")

for dirname, dirlist, filelist in os.walk(dollspath):
    layers = { }

    for fn in filelist:
        realfn = os.path.join(dirname, fn)
        fn = fn.lower()

        if not fn.endswith(".spng"):
            continue

        m = re.match(r'(([^_]+_[^_]+_[^_]+)(_[^-+]+)?([-+][\d.]+))(\.spng)', fn)

        if not m:
            print "Ignoring: ", fn
            continue

        layers.setdefault(m.group(2), [ ]).append((
            float(m.group(4)),
            realfn))

    for layer, imlist in layers.iteritems():
        print layer

        imlist.sort()
        imlist.reverse()

        im = None

        for order, fn in imlist:

            newim = read_spng(fn)

            if im is not None:
                im.paste(newim, None, newim)
            else:
                im = newim

        r, g, b, a = im.split()

        im = im.crop(a.getbbox())
        im.thumbnail((100, 100), Image.ANTIALIAS)

        im.save(os.path.join(dirname, layer + ".tpng"), "PNG")
    
